<?php
declare(strict_types=1);

/**
 * Passbolt ~ Open source password manager for teams
 * Copyright (c) Passbolt SA (https://www.passbolt.com)
 *
 * Licensed under GNU Affero General Public License version 3 of the or any later version.
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright     Copyright (c) Passbolt SA (https://www.passbolt.com)
 * @license       https://opensource.org/licenses/AGPL-3.0 AGPL License
 * @link          https://www.passbolt.com Passbolt(tm)
 * @since         2.0.0
 */
namespace App\Model\Table;

use App\Model\Entity\Role;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * Roles Model
 *
 * @property \App\Model\Table\UsersTable|\Cake\ORM\Association\HasMany $Users
 * @method \App\Model\Entity\Role get($primaryKey, ?array $options = [])
 * @method \App\Model\Entity\Role newEntity($data = null, ?array $options = [])
 * @method \App\Model\Entity\Role[] newEntities(array $data, ?array $options = [])
 * @method \App\Model\Entity\Role|bool save(\Cake\Datasource\EntityInterface $entity, ?array $options = [])
 * @method \App\Model\Entity\Role patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, ?array $options = [])
 * @method \App\Model\Entity\Role[] patchEntities($entities, array $data, ?array $options = [])
 * @method \App\Model\Entity\Role findOrCreate($search, callable $callback = null, ?array $options = [])
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class RolesTable extends Table
{
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('roles');
        $this->setDisplayField('name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('ControllerLogs', [
            'foreignKey' => 'role_id',
        ]);
        $this->hasMany('Users', [
            'foreignKey' => 'role_id',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->scalar('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('name')
            ->requirePresence('name', 'create')
            ->notEmptyString('name')
            ->add('name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table']);

        $validator
            ->scalar('description')
            ->allowEmptyString('description');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
        $rules->add($rules->isUnique(['name']));

        return $rules;
    }

    /**
     * Check if a role name is valid
     *
     * @param string $roleName for example 'admin' or 'user'
     * @return bool true if whitelisted
     */
    public function isValidRoleName(string $roleName)
    {
        $allowedRoleNames = [Role::GUEST, Role::USER, Role::ADMIN, Role::ROOT];

        return in_array($roleName, $allowedRoleNames);
    }

    /**
     * Get a role id by providing its name
     *
     * @param string $roleName such as "admin" or "user"
     * @throws \InvalidArgumentException if the role name is not whitelisted
     * @return mixed|null
     */
    public function getIdByName(string $roleName)
    {
        if (!$this->isValidRoleName($roleName)) {
            $msg = __('The role name should be from the list of allowed role names');
            throw new \InvalidArgumentException($msg);
        }
        $role = $this->find('all')
            ->where(['name' => $roleName])
            ->first();
        if (empty($role)) {
            return null;
        }

        return $role->id;
    }
}
